/**********************************************************************
 * 			MEKATRONIX Copyright 1998 		      *
 * Title        motorme.c  (Motor Drivers for ME11)                *
 * Programmer   Keith L. Doty                      *
 * Date         May 09, 1999                                        *
 * 				                                                     *
 * Version      1  
     Changed some names, set PACTL bits with one command, cleaned up.
 *                                                                    *
 * Description                                                        *
 * Left motor is motor 1 (PA6) and right motor is motor 0 (PA5).      *
 * For efficient use of output compares, OC5 is used to modulate pins *
 * both PA5 and PA6. 		      *
 **********************************************************************/


/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>
/************************************************************************/


/**************************** Constants *********************************/
/*Full period for a motor in clock cycles. A clock cycle is 0.5 microseconds*/
#define MOTOR_HPERIOD 20000

/* One percent of full motor period in clock cycles*/
#define PERIOD_1PC 396   // 396*100 = 39,600 is less than 2*20,000 = 40,000   
						 // 100% duty cycle does not fill 1 period!
						 // Avoids some subtle interrupt timing problems at 100%
#define LEFT_MOTOR 1
#define RIGHT_MOTOR 0

#define PA5_PA6_ONE 0x60
#define PA5_ONE 0x20
#define PA6_ONE 0x40

#define motorme_hand   TOC5_isr
/************************************************************************/


/*************************** Prototypes *********************************/
#pragma interrupt_handler TOC5_isr
void init_motorme(void);
void motorme(int, int);
void motorme_pwm(void);
void motorme_hand(void);
/************************************************************************/


/***************************** Globals **********************************/
unsigned int duty_cycle[2];       /* duty_cycle[] is the on-time for each motor*/
char signal_state, signal_state_mod, pa_edge[3];
unsigned int pw_time[3];
/************************************************************************/



void init_motorme(void)
/************************************************************************
 * Function: Initializes all necessary variables for the motors.        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:   duty_cycle[], signal_state, motormask[], current_duty_cycle       *
 *   Registers: TOC5, TMSK1, TCTL1                               *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  int i;
  
  INTR_OFF();
  CLEAR_BIT(TCTL1,0x03);          /* Interrupt will not affect OC5 pin */

  TOC5 = 0;                       
  PORTA = 0;
  SET_BIT(DDRD, 0x30);		//Enable motor direction control

  duty_cycle[0] = duty_cycle[1] =  0;      /* Motors start turned off */
  signal_state = 0;
  
  signal_state_mod = 1;
  
  for(i=0; i<3; i++)
  {
   pa_edge[i]= 0;
   pw_time[i] = MOTOR_HPERIOD + MOTOR_HPERIOD;
  }
  
  SET_BIT(TMSK1,0x08);            /* Enable OC5 interrupt */
  INTR_ON();
  
}/*end init_motorme*/


void motorme(int index, int per_cent_duty_cycle)
/************************************************************************
 * Function description:						*
 *   Sets duty cycle and direction of motor specified by index.	    	*
 *				                                        *
 * Returns:                                                           	*
 *                                                                    	*
 * Inputs                                                             	*
 *   Parameters: index: 0 = left motor, 1 = right motor               	*
 *		     -100 <= per_cent_duty_cycle<= 100%		        *
 *   Globals:    None                                                 	*
 *   Registers:  None                                                 	*
 * Outputs                                                            	*
 *   Parameters: None                                                 	*
 *   Globals:    duty_cycle[index]					*
        0 <= duty_cycle[index]<= PERIOD (Typically, PERIOD = 65,500)  	*
 *   Registers:  None                                                 	*
 * Functions called: None                                             	*
 * Notes: Checks for proper input bounds. 			 	*
 ************************************************************************/
{
  if (per_cent_duty_cycle < 0)
     {
        per_cent_duty_cycle = -per_cent_duty_cycle; /* Make positive */
     /* Set negative direction of motors */
  	    if (index == RIGHT_MOTOR) CLEAR_BIT(PORTD,0x20); /*Right motor*/
        if (index == LEFT_MOTOR) CLEAR_BIT(PORTD,0x10); /*Left motor*/
     }
  else
     {
     /* Set positive direction of motors */
        if (index == RIGHT_MOTOR) SET_BIT(PORTD, 0x20); /*Right motor*/
        if (index == LEFT_MOTOR) SET_BIT(PORTD, 0x10); /*Left motor*/
     }

/* At this point per_cent_duty_cycle must be a positive number less
 * than 100. If not make it so.
 */
  if (per_cent_duty_cycle > 100) per_cent_duty_cycle = 100;
  duty_cycle[index] = per_cent_duty_cycle*PERIOD_1PC;
  motorme_pwm();

}
/*********************** End motor ***********************************/




void motorme_pwm(void)
//Assumption: Pulse Width in multiples of 396 (1 percent duty cycle)
/************************************************************************
 * Function: Interrupt handler for motor signals                        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    duty_cycle[], signal_state, motormask[], current_duty_cycle      *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    current_duty_cycle                                     *
 *   Registers:  TOC5, PORTA, TFLG1      				                *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  char odd, index;
  unsigned int pwidth0, pwidth1;
  
/*
   Six state sequences:
   1. Neither pulse zero, PA6 longer
   2. Neither pulse zero, PA5 longer
   3. Both pulses equal
   4. PA5 zero pulse, PA6 not zero
   5. PA6 zero pulse, PA5 not zero
   6. Both pulses zero  
*/

//Disable OC5 while setting up PWM data for the interrupt handler

  CLEAR_BIT(TMSK1,0x08);    //Now disable interrupt
  CLEAR_FLAG(TFLG1,0x08);   // Clear OC5I flag, Zen move

//Just to make sure! Stop PWM sequence correctly!
  PORTA = 0;   
  signal_state = 0;

	
  pwidth0 = duty_cycle[0];
  pwidth1 = duty_cycle[1];
 
  if((pwidth0 != 0) && (pwidth1 != 0)) //PA5 & PA6 non-zero
  {
    pa_edge[0] = PA5_PA6_ONE;	//Turn both motor pulses on
    
    if(pwidth0 < pwidth1)
    {
      pw_time[0] = pwidth0;			//PA5 shorter
      
      pa_edge[1] = PA5_ONE;	//Turn PA5 off and leave PA6 on
	  pw_time[1] = pwidth1-pwidth0;
	  
	  pa_edge[2] = PA6_ONE;	//Turn PA6 off and time out period
	  pw_time[2] = MOTOR_HPERIOD - pwidth1 + MOTOR_HPERIOD;

	    
      signal_state_mod = 3;
    }
    else   
    if(pwidth1 < pwidth0)
    {
      pw_time[0] = pwidth1;			//PA6 shorter
      
      pa_edge[1] = PA6_ONE;			//Turn PA6 off and leave PA5 on
	  pw_time[1] = pwidth0-pwidth1;
	  
	  pa_edge[2] = PA5_ONE; 			//Turn PA5 off and time out period
	  pw_time[2] = MOTOR_HPERIOD - pwidth0 + MOTOR_HPERIOD;

      signal_state_mod = 3;
    }
    else 
    if((pwidth1 - pwidth0) == 0)
    {
      pw_time[0] = pwidth0;		//PA5 & PA6 same
      
      pa_edge[1] = PA5_PA6_ONE;	//Turn PA5 and PA6 off and time out period
	  pw_time[1] = MOTOR_HPERIOD - pwidth0 + MOTOR_HPERIOD;
	  
	  signal_state_mod = 2;
    }
  }//end of both pulse widths not zero
  else          
  if((pwidth0 == 0) && (pwidth1 != 0))//Only PA6 non-zero
  {
    pa_edge[0] = PA6_ONE;
    pw_time[0] = pwidth1;
    
	pa_edge[1] = PA6_ONE;	//Turn PA6 off and time out period
	pw_time[1] = MOTOR_HPERIOD - pwidth1 + MOTOR_HPERIOD;

    signal_state_mod = 2;
  }
  else
  if((pwidth1 == 0) && (pwidth0 != 0))//Only PA5 non-zero
  {
    pa_edge[0] = PA5_ONE;
    pw_time[0] = pwidth0;
    
    pa_edge[1] = PA5_ONE; //Turn PA5 off and time out period
	pw_time[1] = MOTOR_HPERIOD - pwidth0 + MOTOR_HPERIOD;

    signal_state_mod = 2;
  }
  else 
  if((pwidth1 == 0) && (pwidth0 == 0))//Both zero
  {
    pa_edge[0] = 0x00;
    pw_time[0] = MOTOR_HPERIOD + MOTOR_HPERIOD;
    
	signal_state_mod = 1;
  }
  
    
  SET_BIT(TMSK1,0x08);            /* Enable OC5 interrupt */

 
}//End of motorme_pwm


void motorme_hand(void)
{
// signal_state starts at 0

  PORTA ^= *(pa_edge + signal_state);
  TOC5 += *(pw_time + signal_state);

  signal_state = (signal_state +1) % signal_state_mod;
  
  CLEAR_FLAG(TFLG1,0x08);          // Clear OC5I flag

}//end of motorme_hand
