/**********************************************************************
 * 			MEKATRONIX Copyright 1998 		      *
 * Title        motorme.c  (Motor Drivers for ME11)                *
 * Programmer   Keith L. Doty                      *
 * Date         May 09, 1999                                        *
 * 				                                                     *
 * Version      1  
     Changed some names, set PACTL bits with one command, cleaned up.
 *                                                                    *
 * Description                                                        *
 * Left motor is motor 1 (PA6) and right motor is motor 0 (PA5).      *
 * For efficient use of output compares, OC5 is used to modulate pins *
 * both PA5 and PA6. 		      *
 **********************************************************************/


/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>
/************************************************************************/


/**************************** Constants *********************************/
/*Full period for a motor in clock cycles. A clock cycle is 0.5 microseconds*/
#define MOTOR_HPERIOD 20000

/* One percent of full motor period in clock cycles*/
#define PERIOD_1PC 400   

#define LEFT_MOTOR 1
#define RIGHT_MOTOR 0

#define PA5_PA6_ONE 0x60
#define PA5_ONE 0x20
#define PA6_ONE 0x40

#define motorme_hand   TOC5_isr
/************************************************************************/


/*************************** Prototypes *********************************/
#pragma interrupt_handler TOC5_isr
void init_motorme(void);
void motorme(int, int);
void motorme_hand(void);
/************************************************************************/


/***************************** Globals **********************************/
unsigned int duty_cycle[2];       /* duty_cycle[] is the on-time for each motor*/
char signal_state;
unsigned int motor_period_end, delta0, delta1;
/************************************************************************/



void init_motorme(void)
/************************************************************************
 * Function: Initializes all necessary variables for the motors.        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:   duty_cycle[], signal_state, motormask[], current_duty_cycle       *
 *   Registers: TOC5, TMSK1, TCTL1                               *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{

  INTR_OFF();
  CLEAR_BIT(TCTL1,0x03);          /* Interrupt will not affect OC5 pin */

  TOC5 = 0;                       
  PORTA = 0;
  SET_BIT(DDRD, 0x30);		//Enable motor direction control

  duty_cycle[0] = duty_cycle[1] =  0;      /* Motors start turned off */
  signal_state = 0;

  SET_BIT(TMSK1,0x08);            /* Enable OC5 interrupt */
  INTR_ON();
}/*end init_motorme*/


void motorme(int index, int per_cent_duty_cycle)
/************************************************************************
 * Function description:						*
 *   Sets duty cycle and direction of motor specified by index.	    	*
 *				                                        *
 * Returns:                                                           	*
 *                                                                    	*
 * Inputs                                                             	*
 *   Parameters: index: 0 = left motor, 1 = right motor               	*
 *		     -100 <= per_cent_duty_cycle<= 100%		        *
 *   Globals:    None                                                 	*
 *   Registers:  None                                                 	*
 * Outputs                                                            	*
 *   Parameters: None                                                 	*
 *   Globals:    duty_cycle[index]					*
        0 <= duty_cycle[index]<= PERIOD (Typically, PERIOD = 65,500)  	*
 *   Registers:  None                                                 	*
 * Functions called: None                                             	*
 * Notes: Checks for proper input bounds. 			 	*
 ************************************************************************/
{
  if (per_cent_duty_cycle < 0)
     {
        per_cent_duty_cycle = -per_cent_duty_cycle; /* Make positive */
     /* Set negative direction of motors */
  	    if (index == RIGHT_MOTOR) CLEAR_BIT(PORTD,0x20); /*Right motor*/
        if (index == LEFT_MOTOR) CLEAR_BIT(PORTD,0x10); /*Left motor*/
     }
  else
     {
     /* Set positive direction of motors */
        if (index == RIGHT_MOTOR) SET_BIT(PORTD, 0x20); /*Right motor*/
        if (index == LEFT_MOTOR) SET_BIT(PORTD, 0x10); /*Left motor*/
     }

/* At this point per_cent_duty_cycle must be a positive number less
 * than 100. If not make it so.
 */
  if (per_cent_duty_cycle > 100) per_cent_duty_cycle = 100;
  duty_cycle[index] = per_cent_duty_cycle*PERIOD_1PC;

}
/*********************** End motor ***********************************/




void motorme_hand(void)
//Assumption: Pulse Width in multiples of 400 (1 percent duty cycle)
/************************************************************************
 * Function: Interrupt handler for motor signals                        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    duty_cycle[], signal_state, motormask[], current_duty_cycle      *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    current_duty_cycle                                          *
 *   Registers:  TOC5, PORTA, TFLG1      				                *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  char odd, index;
  unsigned int pwidth0, pwidth1;
  
  /*
 Control is passed alternately between the motors each half period
   signal_state = 0  -> Turn on  PA5 (motor0) and/or PA6 (motor1) when not 0
   signal_state = 1  -> Turn off PA5 continue PA6
   signal_state = 2  -> Turn off PA6 continue PA5
   signal_state = 3  -> Turn off PA6 and time out rest of period
   signal_state = 4  -> Turn off PA5 and time out rest of period
   signal_state = 5  -> Turn off both pulses and time out rest of period

   signal_state Sequences: 0-5-0, 0-1-3-0, 0-2-4-0, 0-3-0, 0-4-0
*/

  switch(signal_state)
  {
  case 0:
  
    pwidth0 = duty_cycle[0];
    pwidth1 = duty_cycle[1];
    motor_period_end = TOC5 + MOTOR_HPERIOD + MOTOR_HPERIOD;
 
    if((pwidth0 != 0) && (pwidth1 != 0)) //PA5 & PA6 non-zero
    {
      PORTA ^= PA5_PA6_ONE;	//Turn both motor pulses on
  
      if(pwidth0 < pwidth1)
      {
        TOC5 += pwidth0;			//PA5 shorter
        delta1 = pwidth1-pwidth0;
        signal_state = 1;
      }
        
      if(pwidth1 < pwidth0)
      {
        TOC5 += pwidth1;		//PA6 shorter
        delta0 = pwidth0-pwidth1;
        signal_state = 2;
      }
      
      if((pwidth1 - pwidth0) == 0)
      {
        TOC5 += pwidth0;
        signal_state = 5;
      }
    }
        
      
    if((pwidth0 == 0) && (pwidth1 != 0))//Only PA6 non-zero
    {
      PORTA ^= PA6_ONE;
      TOC5 += pwidth1;
      signal_state = 3;
    }
    
    if((pwidth1 == 0) && (pwidth0 != 0))//Only PA5 non-zero
    {
      PORTA ^= PA5_ONE;
      TOC5 += pwidth0;
      signal_state = 4;
    }
      
    if((pwidth1 == 0) && (pwidth0 == 0))//Both zero
    {
      PORTA ^= 0x00;
      TOC5 = motor_period_end;
      signal_state = 0;
    }
 
  break; //End of signal_state = 0
  
  case 1:
   
    PORTA ^= PA5_ONE;	//Turn PA5 off and leave PA6 on
    TOC5 += delta1;
    signal_state = 3;
    
  break; //End of signal_state = 1
  
  case 2: 
        
    PORTA ^= PA6_ONE;	//Turn PA6 off and leave PA5 on
    TOC5 += delta0;
    signal_state = 4;
    
  break; //End of signal_state = 2

  case 3: 
      
    PORTA ^= PA6_ONE;	//Turn PA6 off and time out period
    TOC5 = motor_period_end;
    signal_state = 0;
    
  break; //End of signal_state = 3
    
  case 4: 
      
    PORTA ^= PA5_ONE;	//Turn PA5 off and time out period
    TOC5 += motor_period_end;
    signal_state = 0;
    
  break; //End of signal_state = 4
  
  case 5:
      
    PORTA ^= PA5_PA6_ONE;	//Turn PA5 and PA6 off and time out period
    TOC5 += motor_period_end;
    signal_state = 0;
    
  break; //End of signal_state = 5
  }
       
  CLEAR_FLAG(TFLG1,0x08);          /* Clear OC5I flag */

}/*end of motorme_hand*/
