/**********************************************************************
 * 			MEKATRONIX Copyright 1998 		      *
 * Title        motorme.c  (Motor Drivers for ME11)                *
 * Programmer   Keith L. Doty                      *
 * Date         May 09, 1999                                        *
 * 				                                                     *
 * Version      1  
     Changed some names, set PACTL bits with one command, cleaned up.
 *                                                                    *
 * Description                                                        *
 * Left motor is motor 1 (PA6) and right motor is motor 0 (PA5).      *
 * For efficient use of output compares, OC5 is used to modulate pins *
 * both PA5 and PA6. 		      *
 **********************************************************************/


/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>
/************************************************************************/


/**************************** Constants *********************************/
/*Full period for a motor in clock cycles. A clock cycle is 0.5 microseconds*/
#define MOTOR_HALFPERIOD 20000

/* One percent of full motor period in clock cycles*/
#define PERIOD_1PC 400   

#define LEFT_MOTOR 1
#define RIGHT_MOTOR 0

#define motorme_hand   TOC5_isr
/************************************************************************/


/*************************** Prototypes *********************************/
#pragma interrupt_handler TOC5_isr
void init_motorme(void);
void motorme(int, int);
void motorme_hand(void);
/************************************************************************/


/***************************** Globals **********************************/
unsigned int duty_cycle[2];       /* duty_cycle[] is the on-time for each motor*/
unsigned int current_duty_cycle;
char signal_state, motormask[2];
/************************************************************************/



void init_motorme(void)
/************************************************************************
 * Function: Initializes all necessary variables for the motors.        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:   duty_cycle[], signal_state, motormask[], current_duty_cycle       *
 *   Registers: TOC5, TMSK1, TCTL1                               *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{

  INTR_OFF();
  CLEAR_BIT(TCTL1,0x03);          /* Interrupt will not affect OC5 pin */

  duty_cycle[0] = duty_cycle[1] =  0;      /* Motors start turned off */
  TOC5 = 0;                       
  signal_state = 0;
  current_duty_cycle = 0;
  motormask[0] = 0x20;		/* Drives PA5, right motor*/
  motormask[1] = 0x40;		/* Drives PA6, left motor*/


  SET_BIT(TMSK1,0x08);            /* Enable OC5 interrupt */
  INTR_ON();
}/*end init_motorme*/


void motorme(int index, int per_cent_duty_cycle)
/************************************************************************
 * Function description:						*
 *   Sets duty cycle and direction of motor specified by index.	    	*
 *				                                        *
 * Returns:                                                           	*
 *                                                                    	*
 * Inputs                                                             	*
 *   Parameters: index: 0 = left motor, 1 = right motor               	*
 *		     -100 <= per_cent_duty_cycle<= 100%		        *
 *   Globals:    None                                                 	*
 *   Registers:  None                                                 	*
 * Outputs                                                            	*
 *   Parameters: None                                                 	*
 *   Globals:    duty_cycle[index]					*
        0 <= duty_cycle[index]<= PERIOD (Typically, PERIOD = 65,500)  	*
 *   Registers:  None                                                 	*
 * Functions called: None                                             	*
 * Notes: Checks for proper input bounds. 			 	*
 ************************************************************************/
{
  if (per_cent_duty_cycle < 0)
     {
        per_cent_duty_cycle = -per_cent_duty_cycle; /* Make positive */
     /* Set negative direction of motors */
  	if (index == RIGHT_MOTOR) CLEAR_BIT(PORTD,0x20); /*Right motor*/
        if (index == LEFT_MOTOR) CLEAR_BIT(PORTD,0x10); /*Left motor*/
     }
  else
     {
     /* Set positive direction of motors */
        if (index == RIGHT_MOTOR) SET_BIT(PORTD, 0x20); /*Right motor*/
        if (index == LEFT_MOTOR) SET_BIT(PORTD, 0x10); /*Left motor*/
     }

/* At this point per_cent_duty_cycle must be a positive number less
 * than 100. If not make it so.
 */
  if (per_cent_duty_cycle > 100) per_cent_duty_cycle = 100;
  duty_cycle[index] = per_cent_duty_cycle*PERIOD_1PC;

}
/*********************** End motor ***********************************/




void motorme_hand(void)
/************************************************************************
 * Function: Interrupt handler for motor signals                        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    duty_cycle[], signal_state, motormask[], current_duty_cycle      *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    current_duty_cycle                                          *
 *   Registers:  TOC5, PORTA, TFLG1      				                *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  char odd, index;
  unsigned int pwidth;

/*
 Control is passed alternately between the motors each half period
   signal_state = 0  -> Turn on motor0
   signal_state = 1  -> Turn off motor0
   signal_state = 2  -> Turn on motor1
   signal_state = 3  -> Turn off motor1
*/

  signal_state &= 0x03;         /* Only use last 2 bits */
  index = (signal_state >> 1);  /* index references current motor */

  odd = (signal_state & 0x01);

  pwidth = duty_cycle[index]; /*Load the duty_cycle into pwidth*/
  
  if ((pwidth == 0)&&(!(odd)))
  {
    TOC5 += MOTOR_HALFPERIOD;
    signal_state++;
  }
  else
  {
    if (odd)
      TOC5 += (MOTOR_HALFPERIOD - current_duty_cycle);
    else
      TOC5 += pwidth;

  /* Right motor signal goes on PA5, and left motor signal goes on PA6*/
    PORTA ^= motormask[index];
  }

  current_duty_cycle = pwidth;
  
  signal_state++;
  
  if (signal_state == 4) signal_state = 0;
   
  CLEAR_FLAG(TFLG1,0x08);          /* Clear OC5I flag */

}/*end of motorme_hand*/
