/************************************************************************
 * 			MEKATRONIX Copyright 1998 
 * Title        calsrvtp                                                *
 * Programmer	Keith L. Doty	        				*
 * Date         Sept 17, 1998                                           *
 * Version	1			                		*
 *								        *
 * Description								*
     Calibrates the knot point servo angles of 0, 45, 90, 135 and 180 
     degrees for three servos. Servo counts between knot point angles are
     computed by linear interpolation by the function servodeg(int,float),
     which is included at the end of this program for the user to cut-and
     paste. Futher details are given by comments in the code.
 ************************************************************************/
 
/**************************** Includes **********************************/
#include <tjpbase.h>
#include <stdio.h>
/************************ End of includes *******************************/


/**************************** Prototypes ********************************/

/*Interactive function to generate calibration constants.*/
void calservo(void);

/*
  The function servodeg(,) is attached to this code in order to make clear
  how it relates to the calibration procedure and its use of the global
  calibration constants deg0[3],deg45[3],deg90[3],deg135[3],deg180[3].

 The function servodeg(,) is not actually used here.

  The five calibrated angle values for each servo are used by this function
  to allow a program to move the servos a specified number of degrees by linearly 
  interpolating between the knot points at 0, 45, 90, 135, and 180 degrees.

  In your servo applications, simply, copy this function and the global variables
  deg0[3],deg45[3],deg90[3],deg135[3],deg180[3] into your code.
*/
void servodeg(int, float);

/************************ End of Prototypes *****************************/

/**************************** Globals ***********************************/

/*
  Initial calibration angle counts for three servos. These values are nominal
  only and should be changed by the user using this calibration routine.

  The program outputs a declaration segment just like the one below. The
  program generated segment can be cut-and-pasted to your application programs
  using the same servos.

  If you change out a servo, you should calibrate it and generate a (new) set
  of calibration angles for it.

*/

/*
   Array deg45[i] (i=0,1,2) stores the count for servo_i that moves the servo
   to a 45 degree angle. Similarly for angles specified in the names of deg0,
   deg90, deg135, deg180.
*/

int deg0[3] ={1120,1120,1120};
int deg45[3] ={1930,1930,1930};
int deg90[3] ={2700,2700,2700};
int deg135[3] ={3500,3500,3500};
int deg180[3] ={4400,4400,4400};

char cntrchar;

/**************************** End of Globals ****************************/


void main(void)
/****************************** Main ***********************************/
{
  int i;

  init_analog();
  init_clocktjp();
  init_serial();
  init_servotjp();
  init_clocktjp();

  calservo();

/* Print array of angle calibration constants determined by calservo. */
  printf("\n\nCurrent positions for five knot angles of all three servos.\n\n");
 
  printf("Cut-and-Paste this calibration data from Hyperterm output\n"
         "and declare as global in programs using servos\n\n");
  printf("int deg0[3] = {%d,%d,%d};\n",  deg0[0],  deg0[1],   deg0[2]);
  printf("int deg45[3] = {%d,%d,%d};\n", deg45[0], deg45[1],  deg45[2]);
  printf("int deg90[3] = {%d,%d,%d};\n", deg90[0], deg90[1],  deg90[2]);
  printf("int deg135[3] = {%d,%d,%d};\n",deg135[0],deg135[1], deg135[2]);
  printf("int deg180[3] = {%d,%d,%d};\n",deg180[0],deg180[1], deg180[2]);
   
}
/**************************** End of Main ******************************/


 
void calservo(void)
/*
  Calibrate the servos, i.e., determine the fifteen numbers deg0[3], deg45[3],
   deg90[3], deg135[3] and deg180[3]. The program walks you through interactively.
*/
{
  int nserv, position, angle_sel;

/*Coefficients for converting angle to counts*/
  float cnt45[3], cnt90[3], cnt135[3], cnt180[3];

/* VT100 clear screen */ 
  char clear[]= "\x1b\x5B\x32\x4A\x04"; 

/* VT100 position cursor at (x,y) = (3,12) command is "\x1b[3;12H"*/
  char place[]= "\x1b[1;1H";	/*Home*/

  cntrchar = 'x';

while(!(cntrchar =='n'))
  {
  
  printf("%s", clear);  
  printf("%s", place);

  printf("		SERVO ANGLE CALIBRATION ROUTINE\n"
         "                    by Keith L. Doty\n\n");
 
  printf("Select servo you wish to calibrate (0, 1, 2):  ");
  nserv= read_int();

  printf("\nThe servo will move to nominal zero angle\n\n");
  position = deg0[nserv];

  servo(nserv, position);

  printf("After you select an angle below:\n");
  printf("Use characters 'q''w''e''r''t' to increment servo position by 1000,100,10,5,1\n");
  printf("Use characters 'a''s''d''f''g' to decrement servo position by 1000,100,10,5,1\n\n");
  printf("When the servo is at the selected angle, type character 'p'\n"
         "This will calibrate the current position to the selected angle.\n");

  cntrchar = 'x';

  while(!(cntrchar=='n'))
  {

  printf("\x1b[15;1H");
  printf("Select angle with integer:\n"
       "  1) 0    2) 45    3) 90    4) 135   5) 180   6) Skip   ?\b");

  angle_sel = read_int();

  cntrchar = 'x';

  if((0<angle_sel)&&(angle_sel<6))
  {
  while(!(cntrchar=='p'))
    {  
      cntrchar = getchar();
      switch(cntrchar)
      {
      case 'q': position += 1000; break;
      case 'w': position += 100; break;
      case 'e': position += 10; break;
      case 'r': position += 5; break;
      case 't': position += 1; break;
      case 'a': position -= 1000; break;
      case 's': position -= 100; break;
      case 'd': position -= 10; break;
      case 'f': position -= 5; break;
      case 'g': position -= 1; break;
      }
      servo(nserv, position);
      wait(40);

    }/*end while(!'p')*/

    switch(angle_sel)
    {
    case 1: deg0[nserv]=position; break;
    case 2: deg45[nserv]=position; break;
    case 3: deg90[nserv]=position; break;
    case 4: deg135[nserv]=position; break;
    case 5: deg180[nserv]=position; break;
    }

    }/*end if*/
    else cntrchar = 'n';
 /*
   Compute coefficients cntxx of piecewise linear approximation to position
   versus angle curve.
 */
    cnt45[nserv] = (deg45[nserv]-deg0[nserv])/45.0;
    cnt90[nserv] = (deg90[nserv]-deg45[nserv])/45.0;
    cnt135[nserv] = (deg135[nserv]-deg90[nserv])/45.0;
    cnt180[nserv] = (deg180[nserv]-deg135[nserv])/45.0;

    }/*end while(!'n')*/

    printf("\n\nCurrent positions for five knot angles of servo %d\n",nserv);
    printf("0 deg\t45 deg\t90 deg\t135 deg\t180 deg\n");
    printf("%d\t%d\t%d\t%d\t%d\n",
           deg0[nserv],deg45[nserv],deg90[nserv],
           deg135[nserv],deg180[nserv]);

    printf("\nCurrent conversion factors for servo %d\n",nserv);
    printf("45 CNT\t\t90 CNT\t\t135 CNT\t\t180 CNT\n");
    printf("%f\t\t%f\t%f\t%f\n",
           cnt45[nserv],cnt90[nserv],
           cnt135[nserv],cnt180[nserv]);

    printf("Select another servo or continue with current selection?\n"
           " ('n' for no, any other key for yes): ");
    cntrchar = getchar();


  }/*end outer while*/

}/*end calservo*/
/*************************************************************************/

void servodeg(int nserv, float deg)
/*
  The five calibrated angle values for each servo are used by this function
  to allow a program to move the servos a specified number of degrees by linearly 
  interpolating between the knot points at 0, 45, 90, 135, and 180 degrees.

  In your servo applications, simply, copy this function and the global variables
  deg0[3],deg45[3],deg90[3],deg135[3],deg180[3] into your code.
*/

{
  float count;

  if((0<=deg)&&(deg<45))
      count = deg0[nserv] + deg*(deg45[nserv]-deg0[nserv])/45.0;
  if((45<=deg)&&(deg<90))
      count = deg45[nserv] + (deg-45)*(deg90[nserv]-deg45[nserv])/45.0;
  if((90<=deg)&&(deg<135))
      count = deg90[nserv] + (deg-90)*(deg135[nserv]-deg90[nserv])/45.0;
  if((135<=deg)&&(deg<180))
      count = deg135[nserv] + (deg-135)*(deg180[nserv]-deg135[nserv])/45.0;
  if(180<=deg)
      count = deg180[nserv];

  if(deg>=0) servo(nserv, (int)count);  /*Move the servo*/

}/*end servodeg*/
/*************************************************************************/

